library IEEE;
use IEEE.std_logic_1164.all;
use ieee.numeric_std.all;
use IEEE.std_logic_signed.all;

entity FSL_S2M is
  generic (
    C_DWIDTH      : integer := 32
  );
  port (
    -- Global signals
    Clk   : in std_logic;
    Reset : in std_logic;

    -- Slave FSL Signals
    FSL_S_Clk     : out std_logic;
    FSL_S_Read    : out std_logic;
    FSL_S_Data    : in  std_logic_vector(0 to C_DWIDTH-1);
    FSL_S_Control : in  std_logic;
    FSL_S_Exists  : in  std_logic;

    -- Master FSL Signals
    FSL_M_Clk     : out std_logic;
    FSL_M_Write   : out std_logic;
    FSL_M_Data    : out std_logic_vector(0 to C_DWIDTH-1);
    FSL_M_Control : out std_logic;
    FSL_M_Full    : in  std_logic
    );

end entity FSL_S2M;

library unisim;
use unisim.all;

architecture IMP of FSL_S2M is

    signal bRight: std_logic;
    signal nData: std_logic_vector(15 downto 0);

    type state is (INIT, IDLE, READ, WRITE);
    signal op: state;

begin  -- architecture IMP

    process(Clk, Reset)
    begin
        if (Reset = '1') then
            op <= INIT;
        elsif (Clk'event and Clk = '1') then
            case op is
                -- initialize variables
                when INIT =>
                    bRight <= '0';
                    FSL_S_Read <= '0';
                    FSL_M_Write <= '0';
                    FSL_M_Data <= "00000000000000000000000000000000";
                    FSL_M_Control <= '0';
                    op <= IDLE;
                -- wait for data in input FIFO
                when IDLE =>
                    FSL_S_Read <= '0';
                    FSL_M_Write <= '0';
                    if (FSL_S_Exists = '1') then
                        op <= READ;
                        if (bRight = '0') then
                            nData <= FSL_S_Data(16 to 31);
                        end if;
                    else
                        op <= IDLE;
                    end if;
                -- read data from input FIFO and write out
                -- every other input sample
                when READ =>
                    FSL_S_Read <= '1';
                    if (bRight = '0') then
                        op <= WRITE;
                        FSL_M_Data(16 to 31) <= nData;
                        bRight <= '1';
                    else
                        op <= IDLE;
                        bRight <= '0';
                    end if;
                -- write data to output FIFO
                when WRITE =>
                    FSL_S_Read <= '0';
                    if (FSL_M_Full = '0') then
                        FSL_M_Write <= '1';
                        op <= IDLE;
                    else
                        op <= WRITE;
                    end if;
            end case;
        end if;
    end process;

end architecture IMP;
